@@ -117,12 +117,13 @@ extern struct ifqueue   ipintrq;		/* ip packet input queue */
 /* END OF INCLUDES */
 
 #if !defined(lint)
-static const char rcsid[] = "@(#)$Id: ip_auth.c,v 2.73.2.5 2005/06/12 07:18:14 darrenr Exp $";
+static const char rcsid[] = "@(#)$Id: ip_auth.c,v 2.73.2.12 2006/03/17 22:01:11 darrenr Exp $";
 #endif
 
 
-#if SOLARIS
+#if SOLARIS && defined(_KERNEL)
 extern kcondvar_t ipfauthwait;
+extern struct pollhead iplpollhead[IPL_LOGSIZE];
 #endif /* SOLARIS */
 #if defined(linux) && defined(_KERNEL)
 wait_queue_head_t     fr_authnext_linux;
@@ -339,17 +340,15 @@ fr_info_t *fin;
 	}
 #endif
 #if SOLARIS && defined(_KERNEL)
+	COPYIFNAME(fin->fin_ifp, fra->fra_info.fin_ifname);
 	m->b_rptr -= qpi->qpi_off;
 	fr_authpkts[i] = *(mblk_t **)fin->fin_mp;
 	fra->fra_q = qpi->qpi_q;	/* The queue can disappear! */
+	fra->fra_m = *fin->fin_mp;
+	fra->fra_info.fin_mp = &fra->fra_m;
 	cv_signal(&ipfauthwait);
+	pollwakeup(&iplpollhead[IPL_LOGAUTH], POLLIN|POLLRDNORM);
 #else
-# if defined(BSD) && !defined(sparc) && (BSD >= 199306)
-	if (!fin->fin_out) {
-		ip->ip_len = htons(ip->ip_len);
-		ip->ip_off = htons(ip->ip_off);
-	}
-# endif
 	fr_authpkts[i] = m;
 	WAKEUP(&fr_authnext,0);
 #endif
@@ -362,15 +361,15 @@ caddr_t data;
 ioctlcmd_t cmd;
 int mode;
 {
+	frauth_t auth, *au = &auth, *fra;
+	int i, error = 0, len;
+	char *t;
 	mb_t *m;
 #if defined(_KERNEL) && !defined(MENTAT) && !defined(linux) && \
     (!defined(__FreeBSD_version) || (__FreeBSD_version < 501000))
 	struct ifqueue *ifq;
 	SPL_INT(s);
 #endif
-	frauth_t auth, *au = &auth, *fra;
-	int i, error = 0, len;
-	char *t;
 
 	switch (cmd)
 	{
@@ -399,10 +398,14 @@ int mode;
 	case SIOCAUTHW:
 fr_authioctlloop:
 		error = fr_inobj(data, au, IPFOBJ_FRAUTH);
+		if (error != 0)
+			break;
 		READ_ENTER(&ipf_auth);
 		if ((fr_authnext != fr_authend) && fr_authpkts[fr_authnext]) {
 			error = fr_outobj(data, &fr_auth[fr_authnext],
 					  IPFOBJ_FRAUTH);
+			if (error != 0)
+				break;
 			if (auth.fra_len != 0 && auth.fra_buf != NULL) {
 				/*
 				 * Copy packet contents out to user space if
@@ -416,11 +419,12 @@ fr_authioctlloop:
 				for (t = auth.fra_buf; m && (len > 0); ) {
 					i = MIN(M_LEN(m), len);
 					error = copyoutptr(MTOD(m, char *),
-							  t, i);
+							   &t, i);
 					len -= i;
 					t += i;
 					if (error != 0)
 						break;
+					m = m->m_next;
 				}
 			}
 			RWLOCK_EXIT(&ipf_auth);
@@ -473,10 +477,8 @@ fr_authioctlloop:
 #endif
 		MUTEX_EXIT(&ipf_authmx);
 		READ_ENTER(&ipf_global);
-		if (error == 0) {
-			READ_ENTER(&ipf_auth);
+		if (error == 0)
 			goto fr_authioctlloop;
-		}
 		break;
 
 	case SIOCAUTHR:
@@ -487,6 +489,7 @@ fr_authioctlloop:
 		WRITE_ENTER(&ipf_auth);
 		i = au->fra_index;
 		fra = fr_auth + i;
+		error = 0;
 		if ((i < 0) || (i >= fr_authsize) ||
 		    (fra->fra_info.fin_id != au->fra_info.fin_id)) {
 			RWLOCK_EXIT(&ipf_auth);
@@ -501,7 +504,11 @@ fr_authioctlloop:
 #ifdef	_KERNEL
 		if ((m != NULL) && (au->fra_info.fin_out != 0)) {
 # ifdef MENTAT
-			error = !putq(fra->fra_q, m);
+			error = ipf_inject(&fra->fra_info);
+			if (error != 0) {
+				FREE_MB_T(m);
+				error = ENOBUFS;
+			}
 # else /* MENTAT */
 #  if defined(linux) || defined(AIX)
 #  else
@@ -521,7 +528,11 @@ fr_authioctlloop:
 				fr_authstats.fas_sendok++;
 		} else if (m) {
 # ifdef MENTAT
-			error = !putq(fra->fra_q, m);
+			error = ipf_inject(&fra->fra_info);
+			if (error != 0) {
+				FREE_MB_T(m);
+				error = ENOBUFS;
+			}
 # else /* MENTAT */
 #  if defined(linux) || defined(AIX)
 #  else
@@ -552,10 +563,6 @@ fr_authioctlloop:
 				fr_authstats.fas_queok++;
 		} else
 			error = EINVAL;
-# ifdef MENTAT
-		if (error != 0)
-			error = EINVAL;
-# else /* MENTAT */
 		/*
 		 * If we experience an error which will result in the packet
 		 * not being processed, make sure we advance to the next one.
@@ -579,7 +586,6 @@ fr_authioctlloop:
 				}
 			}
 		}
-# endif /* MENTAT */
 #endif /* _KERNEL */
 		SPL_X(s);
 		break;
@@ -794,3 +800,9 @@ int fr_authflush()
 
 	return num_flushed;
 }
+
+
+int fr_auth_waiting()
+{
+	return (fr_authnext != fr_authend) && fr_authpkts[fr_authnext];
+}
