@@ -58,7 +58,7 @@ struct file;
 
 #if !defined(lint)
 static const char sccsid[] = "@(#)ip_state.c	1.8 6/5/96 (C) 1993-2000 Darren Reed";
-static const char rcsid[] = "@(#)$Id: ip_scan.c,v 2.40.2.4 2005/08/20 13:48:24 darrenr Exp $";
+static const char rcsid[] = "@(#)$Id: ip_scan.c,v 2.40.2.6 2006/03/26 23:06:49 darrenr Exp $";
 #endif
 
 #ifdef	IPFILTER_SCAN	/* endif at bottom of file */
@@ -84,18 +84,23 @@ int ipsc_matchstr __P((sinfo_t *, char *, int));
 int ipsc_matchisc __P((ipscan_t *, ipstate_t *, int, int, int *));
 int ipsc_match __P((ipstate_t *));
 
+static int	ipsc_inited = 0;
 
 
 int ipsc_init()
 {
 	RWLOCK_INIT(&ipsc_rwlock, "ip scan rwlock");
+	ipsc_inited = 1;
 	return 0;
 }
 
 
 void fr_scanunload()
 {
-	RW_DESTROY(&ipsc_rwlock);
+	if (ipsc_inited == 1) {
+		RW_DESTROY(&ipsc_rwlock);
+		ipsc_inited = 0;
+	}
 }
 
 
@@ -431,6 +436,8 @@ ipstate_t *is;
 		}
 		if (k == 1)
 			isc = lm;
+		if (isc == NULL)
+			return 0;
 
 		/*
 		 * No matches or partial matches, so reset the respective
