@@ -7,11 +7,19 @@ char *name;
 	char *s;
 
 	for (s = name; *s != '\0'; s++)
-		if (!isdigit(*s))
+		if (!ISDIGIT(*s))
 			break;
 	if (*s == '\0')
 		return atoi(name);
 
+#ifdef _AIX51
+	/*
+	 * For some bogus reason, "ip" is 252 in /etc/protocols on AIX 5
+	 */
+	if (!strcasecmp(name, "ip"))
+		return 0;
+#endif
+
 	p = getprotobyname(name);
 	if (p != NULL)
 		return p->p_proto;
