@@ -3,7 +3,7 @@
  *
  * See the IPFILTER.LICENCE file for details on licencing.
  *
- * $Id: printfr.c,v 1.43.2.12 2005/06/12 07:18:42 darrenr Exp $
+ * $Id: printfr.c,v 1.43.2.15 2005/11/14 17:45:06 darrenr Exp $
  */
 
 #include "ipf.h"
@@ -20,7 +20,7 @@ u_32_t *addr, *mask;
 	switch (type)
 	{
 	case FRI_BROADCAST :
-		suffix = "/bcast";
+		suffix = "bcast";
 		break;
 
 	case FRI_DYNAMIC :
@@ -30,15 +30,15 @@ u_32_t *addr, *mask;
 		break;
 
 	case FRI_NETWORK :
-		suffix = "/net";
+		suffix = "net";
 		break;
 
 	case FRI_NETMASKED :
-		suffix = "/netmasked";
+		suffix = "netmasked";
 		break;
 
 	case FRI_PEERADDR :
-		suffix = "/peer";
+		suffix = "peer";
 		break;
 
 	case FRI_LOOKUP :
@@ -190,12 +190,11 @@ ioctlfunc_t	iocfunc;
 		if (*fp->fr_ifnames[2]) {
 			printifname("", fp->fr_ifnames[2],
 				    fp->fr_ifas[2]);
-			putchar(' ');
-
 			if (*fp->fr_ifnames[3]) {
 				printifname(",", fp->fr_ifnames[3],
 					    fp->fr_ifas[3]);
 			}
+			putchar(' ');
 		}
 	}
 
@@ -371,6 +370,35 @@ ioctlfunc_t	iocfunc;
 			if (!(fp->fr_flx & FI_OOW))
 				printf("not ");
 			printf("oow");
+			comma = ",";
+		}
+		if (fp->fr_mflx & FI_MBCAST) {
+			fputs(comma, stdout);
+			if (!(fp->fr_flx & FI_MBCAST))
+				printf("not ");
+			printf("mbcast");
+			comma = ",";
+		}
+		if (fp->fr_mflx & FI_BROADCAST) {
+			fputs(comma, stdout);
+			if (!(fp->fr_flx & FI_BROADCAST))
+				printf("not ");
+			printf("bcast");
+			comma = ",";
+		}
+		if (fp->fr_mflx & FI_MULTICAST) {
+			fputs(comma, stdout);
+			if (!(fp->fr_flx & FI_MULTICAST))
+				printf("not ");
+			printf("mcast");
+			comma = ",";
+		}
+		if (fp->fr_mflx & FI_STATE) {
+			fputs(comma, stdout);
+			if (!(fp->fr_flx & FI_STATE))
+				printf("not ");
+			printf("state");
+			comma = ",";
 		}
 	}
 
@@ -411,8 +439,8 @@ ioctlfunc_t	iocfunc;
 		if (fp->fr_flags & (FR_FRSTRICT)) {
 			printf(" (");
 			if (fp->fr_flags & FR_FRSTRICT)
-				printf(" strict");
-			printf(" )");
+				printf("strict");
+			printf(")");
 				
 		}
 	}
