@@ -5,7 +5,7 @@
  */
 #if !defined(lint)
 static const char sccsid[] = "@(#)ip_fil.c	2.41 6/5/96 (C) 1993-2000 Darren Reed";
-static const char rcsid[] = "@(#)$Id: ipsyncs.c,v 1.5.2.1 2004/10/31 18:46:44 darrenr Exp $";
+static const char rcsid[] = "@(#)$Id: ipsyncs.c,v 1.5.2.2 2006/03/21 16:09:31 darrenr Exp $";
 #endif
 #include <sys/types.h>
 #include <sys/time.h>
@@ -19,7 +19,7 @@ static const char rcsid[] = "@(#)$Id: ipsyncs.c,v 1.5.2.1 2004/10/31 18:46:44 da
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <strings.h>
+#include <string.h>
 #include <unistd.h>
 #include <syslog.h>
 #include <errno.h>
@@ -41,11 +41,12 @@ void usage(const char *progname) {
 		progname);
 }
 
+#if 0
 static void handleterm(int sig)
 {
 	terminate = sig;
-
 }
+#endif
 
 #define BUFFERLEN 1400
 
@@ -130,8 +131,7 @@ char *argv[];
 			goto tryagain;
 		}
 
-		syslog(LOG_INFO, "Established connection to %s",
-		       inet_ntoa(sin.sin_addr));
+		syslog(LOG_INFO, "Listening to %s", inet_ntoa(sin.sin_addr));
 	
 		inbuf = 0;	
 		while (1) {
@@ -223,14 +223,15 @@ moreinbuf:
 			n2 = sizeof(*sh) + len;
 			n3 = write(lfd, buff, n2);
 			if (n3 <= 0) {
-				syslog(LOG_ERR, "Write error: %m");
+				syslog(LOG_ERR, "%s: Write error: %m",
+				       IPSYNC_NAME);
 				goto tryagain;
 			}
 
 			
 			if (n3 != n2) {
-				syslog(LOG_ERR, "Incomplete write (%d/%d)",
-				       n3, n2);
+				syslog(LOG_ERR, "%s: Incomplete write (%d/%d)",
+				       IPSYNC_NAME, n3, n2);
 				goto tryagain;
 			}
 
